from abc import ABCMeta, abstractmethod
from messagebus import Message


class SendMessageAdapter(object):
    __metaclass__ = ABCMeta

    @abstractmethod
    def send_message(self, component, message):
        # type: (str, Message) -> Message
        raise NotImplementedError()



from ._LambdaAdapter import LambdaAdapter
from ._LambdaMessageBus import LambdaMessageBus
from ._LambdaMessageHandler import LambdaMessageHandler
from ._LambdaEventType import LambdaEventType
from ._SendMessageAdapter import SendMessageAdapter
from ._PublishEventAdapter import PublishEventAdapter
from ._UnhandledEventException import UnhandledEventException


__all__ = [
    "LambdaAdapter",
    "LambdaMessageBus",
    "LambdaMessageHandler",
    "LambdaEventType",
    "SendMessageAdapter",
    "PublishEventAdapter",
    "UnhandledEventException"
]



from typing import Dict, Any
from abc import ABCMeta, abstractmethod
from messagebus import Message, Event
from ._LambdaEventType import LambdaEventType


class LambdaAdapter(object):
    __metaclass__ = ABCMeta

    @abstractmethod
    def get_type(self, event, context):
        # type: (Dict, Dict) -> LambdaEventType
        raise NotImplementedError()

    @abstractmethod
    def create_message(self, event, context):
        # type: (Dict, Dict) -> Message
        raise NotImplementedError()

    @abstractmethod
    def create_event(self, event, context):
        # type: (Dict, Dict) -> Event
        raise NotImplementedError()

    @abstractmethod
    def create_reply(self, event, context, message):
        # type: (Dict, Dict, Message) -> Any
        raise NotImplementedError()



from abc import ABCMeta, abstractmethod
from messagebus import Event, Message


class PublishEventAdapter(object):
    __metaclass__ = ABCMeta

    @abstractmethod
    def publish_event(self, event):
        # type: (Event) -> Message
        raise NotImplementedError()



from typing import Optional
from messagebus import MessageBus, Event, Message
from ._SendMessageAdapter import SendMessageAdapter
from ._PublishEventAdapter import PublishEventAdapter


class LambdaMessageBus(MessageBus):
    def __init__(self, send_message_adapter, publish_event_adapter):
        # type: (Optional[SendMessageAdapter], Optional[PublishEventAdapter]) -> None
        self.send_message_adapter = send_message_adapter
        self.publish_event_adapter = publish_event_adapter

        self.reply = None

    def publish_event(self, event):
        # type: (Event) -> Message
        return self.publish_event_adapter.publish_event(event)

    def send_message(self, component, message):
        # type: (unicode, Message) -> Message
        return self.send_message_adapter.send_message(component, message)

    def subscribe(self, subject):
        # type: (unicode) -> None
        raise NotImplementedError()

    def reply_message(self, message, reply_message):
        # type: (Message, Message) -> None
        self.reply = reply_message

    def reply_event(self, event, reply_message):
        # type: (Event, Message) -> None
        self.reply = reply_message

    def get_reply(self):
        # type: () -> Dict
        return self.reply



import logging
from typing import Dict, Callable, Optional

from messagebus import Message, Event

from ._LambdaMessageBus import LambdaMessageBus
from ._LambdaAdapter import LambdaAdapter
from ._LambdaEventType import LambdaEventType
from ._UnhandledEventException import UnhandledEventException
from messagehandler import MessageHandler, MessageHandlerBuilder


logger = logging.getLogger()


class LambdaMessageHandler(object):
    def __init__(self, message_bus, message_handler_builder, lambda_adapter):
        # type: (LambdaMessageBus, MessageHandlerBuilder, LambdaAdapter) -> None
        self.message_bus = message_bus
        self.message_handler_builder = message_handler_builder
        self.lambda_adapter = lambda_adapter

    def set_event_handler(self, event_handler_builder):
        self.message_handler_builder = event_handler_builder

    def handle_events(self, event, context):
        # type: (Dict, Dict) -> Dict
        try:
            lambda_event_type = self.lambda_adapter.get_type(event, context)
            if lambda_event_type == LambdaEventType.event or lambda_event_type == LambdaEventType.sync_event:
                message_bus_event = self.lambda_adapter.create_event(event, context)

                def handle_event_func(event_handler):
                    # type: (MessageHandler) -> None
                    event_handler.handle_event(self.message_bus, message_bus_event)

                self._execute_with_event_handler(handle_event_func, None, message_bus_event)
                if lambda_event_type == LambdaEventType.sync_event:
                    return self.lambda_adapter.create_reply(event, context, self.message_bus.get_reply())
            elif lambda_event_type == LambdaEventType.message:
                message = self.lambda_adapter.create_message(event, context)

                def handle_message_func(message_handler):
                    # type: (MessageHandler) -> None
                    message_handler.handle_message(self.message_bus, message)

                self._execute_with_event_handler(handle_message_func, message, None)

                return self.lambda_adapter.create_reply(event, context, self.message_bus.get_reply())
            else:
                raise Exception("Unhandled event type: {}".format(lambda_event_type))
        except UnhandledEventException:
            logger.exception("Unhandled event: {}".format(event))

    def _execute_with_event_handler(self, func, message, event):
        # type: (Callable[[MessageHandler], None], Optional[Message], Optional[Event]) -> None
        message_handler = None
        try:
            message_handler = self.message_handler_builder.build_message_handler(message, event)

            func(message_handler)
        finally:
            if message_handler is not None:
                self.message_handler_builder.destroy_message_handler(message_handler)
